# frozen_string_literal: true

warn "You are loading 'unicode-display_width/no_string_ext'\n" \
     "Beginning with version 2.0, this is not necessary anymore\n"\
     "You can just require 'unicode-display_width' now and no\n"\
     "string extension will be loaded"

require_relative "../display_width"
